from rosnik import constants

JOURNEY_ID_KEY = f"X-{constants.NAMESPACE}-Journey-Id"
INTERACTION_ID_KEY = f"X-{constants.NAMESPACE}-Interaction-Id"
DEVICE_ID_KEY = f"X-{constants.NAMESPACE}-Device-Id"
